Class: Rubydex::ClassVariable
- Inherits:
-
Declaration
- Object
- Declaration
- Rubydex::ClassVariable
- Defined in:
- ext/rubydex/declaration.c
Instance Method Summary collapse
-
#references ⇒ Array[untyped]
Returns an empty array because variable declarations do not yet support reference lookup.
Methods inherited from Declaration
#definitions, #initialize, #name, #owner, #unqualified_name
Constructor Details
This class inherits a constructor from Rubydex::Declaration
Instance Method Details
#references ⇒ Array[untyped]
Returns an empty array because variable declarations do not yet support reference lookup.
412 413 414 |
# File 'ext/rubydex/declaration.c', line 412 static VALUE rdxr_variable_declaration_references(VALUE self) { return rb_ary_new(); } |