Class: Rubydex::Rules::ParseError

Inherits:
Rubydex::Rule show all
Defined in:
lib/rubydex/rules/parse_error.rb

Overview

A parse error represents invalid Ruby syntax and a program that will fail to execute. For example, a missing end, an unterminated string, a missing parenthesis.

class Foo
^^^^^ Syntax error. Missing end token

Class Method Summary collapse

Methods inherited from Rubydex::Rule

rule_name, severity

Class Method Details

.default_severityObject

: -> singleton(Severity::Base)



19
20
21
# File 'lib/rubydex/rules/parse_error.rb', line 19

def default_severity
  Severity::Error
end