Class: Rubydex::Graph

Inherits:
Object
  • Object
show all
Defined in:
lib/rubydex/graph.rb,
ext/rubydex/graph.c

Overview

The global graph representing all declarations and their relationships for the workspace

Note: this class is partially defined in C to integrate with the Rust backend

Constant Summary collapse

INDEXABLE_EXTENSIONS =
[".rb", ".rake", ".rbs", ".ru"].freeze

Instance Method Summary collapse

Constructor Details

#initialize(workspace_path: nil) ⇒ Graph

: (?workspace_path: String?) -> void



11
12
13
# File 'lib/rubydex/graph.rb', line 11

def initialize(workspace_path: nil)
  self.workspace_path = workspace_path if workspace_path
end

Instance Method Details

#[](fully_qualified_name) ⇒ Rubydex::Declaration?

Returns the declaration for the fully qualified name, or nil when no declaration exists.

Returns:



311
312
313
314
315
316
317
318
319
320
321
322
323
324
325
326
327
328
329
# File 'ext/rubydex/graph.c', line 311

static VALUE rdxr_graph_aref(VALUE self, VALUE key) {
    void *graph;
    TypedData_Get_Struct(self, void *, &graph_type, graph);

    if (TYPE(key) != T_STRING) {
        rb_raise(rb_eTypeError, "expected String");
    }

    const CDeclaration *decl = rdx_graph_get_declaration(graph, StringValueCStr(key));
    if (decl == NULL) {
        return Qnil;
    }

    VALUE decl_class = rdxi_declaration_class_for_kind(decl->kind);
    VALUE argv[] = {self, ULL2NUM(decl->id)};
    free_c_declaration(decl);

    return rb_class_new_instance(2, argv, decl_class);
}

#check_integrityArray[Rubydex::IntegrityFailure]

Returns an array of integrity failures, or an empty array if no issues were found.

Returns:



579
580
581
582
583
584
585
586
587
588
589
590
591
592
593
594
595
596
597
598
599
600
601
# File 'ext/rubydex/graph.c', line 579

static VALUE rdxr_graph_check_integrity(VALUE self) {
    void *graph;
    TypedData_Get_Struct(self, void *, &graph_type, graph);

    size_t error_count = 0;
    const char *const *errors = rdx_check_integrity(graph, &error_count);

    if (errors == NULL) {
        return rb_ary_new();
    }

    VALUE cIntegrityError = rb_const_get(mRubydex, rb_intern("IntegrityFailure"));
    VALUE array = rb_ary_new_capa((long)error_count);

    for (size_t i = 0; i < error_count; i++) {
        VALUE argv[] = {rb_utf8_str_new_cstr(errors[i])};
        VALUE error = rb_class_new_instance(1, argv, cIntegrityError);
        rb_ary_push(array, error);
    }

    free_c_string_array(errors, error_count);
    return array;
}

#complete_expression(nesting, self_receiver:) ⇒ Array[Rubydex::Declaration | Rubydex::Keyword]

Returns completion candidates for an expression context. The nesting array represents the lexical scope stack. The required self_receiver keyword argument overrides the self type; pass nil when the self type is unknown.



704
705
706
707
708
709
710
711
712
713
714
715
716
717
718
719
720
721
722
# File 'ext/rubydex/graph.c', line 704

static VALUE rdxr_graph_complete_expression(int argc, VALUE *argv, VALUE self) {
    VALUE nesting, opts;
    rb_scan_args(argc, argv, "1:", &nesting, &opts);
    rdxi_check_array_of_strings(nesting);

    const char *self_receiver = extract_self_receiver(opts);

    void *graph;
    TypedData_Get_Struct(self, void *, &graph_type, graph);

    size_t nesting_count = RARRAY_LEN(nesting);
    char **converted_nesting = rdxi_str_array_to_char(nesting, nesting_count);

    struct CompletionResult result =
        rdx_graph_complete_expression(graph, (const char *const *)converted_nesting, nesting_count, self_receiver);

    rdxi_free_str_array(converted_nesting, nesting_count);
    return completion_result_to_ruby_array(result, self);
}

#complete_method_argument(name, nesting, self_receiver:) ⇒ Array[Rubydex::Declaration | Rubydex::Keyword | Rubydex::KeywordParameter]

Returns completion candidates inside a method call's argument list. See complete_expression for self_receiver semantics.



775
776
777
778
779
780
781
782
783
784
785
786
787
788
789
790
791
792
793
794
795
# File 'ext/rubydex/graph.c', line 775

static VALUE rdxr_graph_complete_method_argument(int argc, VALUE *argv, VALUE self) {
    VALUE name, nesting, opts;
    rb_scan_args(argc, argv, "2:", &name, &nesting, &opts);

    Check_Type(name, T_STRING);
    rdxi_check_array_of_strings(nesting);

    const char *self_receiver = extract_self_receiver(opts);

    void *graph;
    TypedData_Get_Struct(self, void *, &graph_type, graph);

    size_t nesting_count = RARRAY_LEN(nesting);
    char **converted_nesting = rdxi_str_array_to_char(nesting, nesting_count);

    struct CompletionResult result = rdx_graph_complete_method_argument(
        graph, StringValueCStr(name), (const char *const *)converted_nesting, nesting_count, self_receiver);

    rdxi_free_str_array(converted_nesting, nesting_count);
    return completion_result_to_ruby_array(result, self);
}

#complete_method_call(name, self_receiver:) ⇒ Array[Rubydex::Method]

Returns completion candidates after a method call operator such as foo. The required self_receiver keyword argument is the caller's runtime self type; pass nil when there is no caller context.

Returns:



753
754
755
756
757
758
759
760
761
762
763
764
765
766
# File 'ext/rubydex/graph.c', line 753

static VALUE rdxr_graph_complete_method_call(int argc, VALUE *argv, VALUE self) {
    VALUE name, opts;
    rb_scan_args(argc, argv, "1:", &name, &opts);
    Check_Type(name, T_STRING);

    const char *self_receiver = extract_self_receiver(opts);

    void *graph;
    TypedData_Get_Struct(self, void *, &graph_type, graph);

    struct CompletionResult result =
        rdx_graph_complete_method_call(graph, StringValueCStr(name), self_receiver);
    return completion_result_to_ruby_array(result, self);
}

#complete_namespace_access(name, self_receiver:) ⇒ Array[Rubydex::Declaration]

Returns completion candidates after a namespace access operator such as Foo::. The required self_receiver keyword argument is the caller's runtime self type; pass nil when there is no caller context.

Returns:



731
732
733
734
735
736
737
738
739
740
741
742
743
744
# File 'ext/rubydex/graph.c', line 731

static VALUE rdxr_graph_complete_namespace_access(int argc, VALUE *argv, VALUE self) {
    VALUE name, opts;
    rb_scan_args(argc, argv, "1:", &name, &opts);
    Check_Type(name, T_STRING);

    const char *self_receiver = extract_self_receiver(opts);

    void *graph;
    TypedData_Get_Struct(self, void *, &graph_type, graph);

    struct CompletionResult result =
        rdx_graph_complete_namespace_access(graph, StringValueCStr(name), self_receiver);
    return completion_result_to_ruby_array(result, self);
}

#constant_referencesEnumerator[Rubydex::ConstantReference]

Returns an enumerator that yields constant references lazily.

Returns:



349
350
351
352
353
354
355
356
357
358
359
360
361
362
363
# File 'ext/rubydex/graph.c', line 349

static VALUE rdxr_graph_constant_references(VALUE self) {
    if (!rb_block_given_p()) {
        return rb_enumeratorize_with_size(self, rb_str_new2("constant_references"), 0, NULL,
                                          graph_constant_references_size);
    }

    void *graph;
    TypedData_Get_Struct(self, void *, &graph_type, graph);

    void *iter = rdx_graph_constant_references_iter_new(graph);
    VALUE args = rb_ary_new_from_args(2, self, ULL2NUM((uintptr_t)iter));
    rb_ensure(rdxi_constant_references_yield, args, rdxi_constant_references_ensure, args);

    return self;
}

#declarationsEnumerator[Rubydex::Declaration]

Returns an enumerator that yields all declarations lazily.

Returns:



169
170
171
172
173
174
175
176
177
178
179
180
181
182
# File 'ext/rubydex/graph.c', line 169

static VALUE rdxr_graph_declarations(VALUE self) {
    if (!rb_block_given_p()) {
        return rb_enumeratorize_with_size(self, rb_str_new2("declarations"), 0, NULL, graph_declarations_size);
    }

    void *graph;
    TypedData_Get_Struct(self, void *, &graph_type, graph);

    void *iter = rdx_graph_declarations_iter_new(graph);
    VALUE args = rb_ary_new_from_args(2, self, ULL2NUM((uintptr_t)iter));
    rb_ensure(rdxi_declarations_yield, args, rdxi_declarations_ensure, args);

    return self;
}

#delete_document(uri) ⇒ Rubydex::Document?

Deletes a document and all of its definitions from the graph. Returns the removed document, or nil if it does not exist.

Returns:



428
429
430
431
432
433
434
435
436
437
438
439
440
441
442
# File 'ext/rubydex/graph.c', line 428

static VALUE rdxr_graph_delete_document(VALUE self, VALUE uri) {
    Check_Type(uri, T_STRING);

    void *graph;
    TypedData_Get_Struct(self, void *, &graph_type, graph);
    const uint64_t *uri_id = rdx_graph_delete_document(graph, StringValueCStr(uri));

    if (uri_id == NULL) {
        return Qnil;
    }

    VALUE argv[] = {self, ULL2NUM(*uri_id)};
    free_u64(uri_id);
    return rb_class_new_instance(2, argv, cDocument);
}

#diagnosticsArray[Rubydex::Diagnostic]

Returns diagnostics emitted while indexing or resolving the graph.

Returns:



609
610
611
612
613
614
615
616
617
618
619
620
621
622
623
624
625
626
627
628
629
630
631
632
633
634
635
636
637
638
639
# File 'ext/rubydex/graph.c', line 609

static VALUE rdxr_graph_diagnostics(VALUE self) {
    void *graph;
    TypedData_Get_Struct(self, void *, &graph_type, graph);

    DiagnosticArray *array = rdx_graph_diagnostics(graph);
    if (array == NULL || array->len == 0) {
        if (array != NULL) {
            rdx_diagnostics_free(array);
        }
        return rb_ary_new();
    }

    VALUE diagnostics = rb_ary_new_capa((long)array->len);
    for (size_t i = 0; i < array->len; i++) {
        DiagnosticEntry entry = array->items[i];
        VALUE message = entry.message == NULL ? Qnil : rb_utf8_str_new_cstr(entry.message);
        VALUE rule = rb_str_intern(rb_str_new2(entry.rule));
        VALUE location = rdxi_build_location_value(entry.location);

        VALUE kwargs = rb_hash_new();
        rb_hash_aset(kwargs, ID2SYM(rb_intern("rule")), rule);
        rb_hash_aset(kwargs, ID2SYM(rb_intern("message")), message);
        rb_hash_aset(kwargs, ID2SYM(rb_intern("location")), location);

        VALUE diagnostic = rb_class_new_instance_kw(1, &kwargs, cDiagnostic, RB_PASS_KEYWORDS);
        rb_ary_push(diagnostics, diagnostic);
    }

    rdx_diagnostics_free(array);
    return diagnostics;
}

#document(uri) ⇒ Rubydex::Document?

Returns the document for the URI, or nil if it does not exist.

Returns:



405
406
407
408
409
410
411
412
413
414
415
416
417
418
419
# File 'ext/rubydex/graph.c', line 405

static VALUE rdxr_graph_document(VALUE self, VALUE uri) {
    Check_Type(uri, T_STRING);

    void *graph;
    TypedData_Get_Struct(self, void *, &graph_type, graph);
    const uint64_t *uri_id = rdx_graph_get_document(graph, StringValueCStr(uri));

    if (uri_id == NULL) {
        return Qnil;
    }

    VALUE argv[] = {self, ULL2NUM(*uri_id)};
    free_u64(uri_id);
    return rb_class_new_instance(2, argv, cDocument);
}

#documentsEnumerator[Rubydex::Document]

Returns an enumerator that yields all documents lazily.

Returns:



290
291
292
293
294
295
296
297
298
299
300
301
302
303
# File 'ext/rubydex/graph.c', line 290

static VALUE rdxr_graph_documents(VALUE self) {
    if (!rb_block_given_p()) {
        return rb_enumeratorize_with_size(self, rb_str_new2("documents"), 0, NULL, graph_documents_size);
    }

    void *graph;
    TypedData_Get_Struct(self, void *, &graph_type, graph);

    void *iter = rdx_graph_documents_iter_new(graph);
    VALUE args = rb_ary_new_from_args(2, self, ULL2NUM((uintptr_t)iter));
    rb_ensure(graph_documents_yield, args, graph_documents_ensure, args);

    return self;
}

#encoding=(encoding) ⇒ nil

Sets the encoding used for transforming byte offsets into LSP code unit line and column positions.

Returns:

  • (nil)


463
464
465
466
467
468
469
470
471
472
473
474
475
# File 'ext/rubydex/graph.c', line 463

static VALUE rdxr_graph_set_encoding(VALUE self, VALUE encoding) {
    Check_Type(encoding, T_STRING);

    void *graph;
    TypedData_Get_Struct(self, void *, &graph_type, graph);

    char *encoding_string = StringValueCStr(encoding);
    if (!rdx_graph_set_encoding(graph, encoding_string)) {
        rb_raise(rb_eArgError, "invalid encoding `%s` (should be utf8, utf16 or utf32)", encoding_string);
    }

    return Qnil;
}

#exclude_paths(paths) ⇒ nil

Excludes the paths from file discovery during indexing.

Returns:

  • (nil)


803
804
805
806
807
808
809
810
811
812
813
814
815
816
817
# File 'ext/rubydex/graph.c', line 803

static VALUE rdxr_graph_exclude_paths(VALUE self, VALUE paths) {
    Check_Type(paths, T_ARRAY);
    rdxi_check_array_of_strings(paths);

    size_t length = RARRAY_LEN(paths);
    char **converted_paths = rdxi_str_array_to_char(paths, length);

    void *graph;
    TypedData_Get_Struct(self, void*, &graph_type, graph);

    rdx_graph_exclude_paths(graph, (const char **)converted_paths, length);
    rdxi_free_str_array(converted_paths, length);

    return Qnil;
}

#excluded_pathsArray[String]

Returns the paths currently excluded from file discovery.

Returns:

  • (Array[String])


825
826
827
828
829
830
831
832
833
834
835
836
837
838
839
840
841
842
843
# File 'ext/rubydex/graph.c', line 825

static VALUE rdxr_graph_excluded_paths(VALUE self) {
    void *graph;
    TypedData_Get_Struct(self, void*, &graph_type, graph);

    size_t out_count = 0;
    const char *const *results = rdx_graph_excluded_paths(graph, &out_count);

    if (results == NULL) {
        return rb_ary_new();
    }

    VALUE array = rb_ary_new_capa((long)out_count);
    for (size_t i = 0; i < out_count; i++) {
        rb_ary_push(array, rb_utf8_str_new_cstr(results[i]));
    }

    free_c_string_array(results, out_count);
    return array;
}

#fuzzy_search(*queries) ⇒ Enumerator[Rubydex::Declaration]

Returns an enumerator that yields declarations whose name matches any of the queries fuzzily.

Returns:



229
230
231
232
233
234
235
236
237
238
239
240
241
242
243
244
245
246
247
# File 'ext/rubydex/graph.c', line 229

static VALUE rdxr_graph_fuzzy_search(int argc, VALUE *argv, VALUE self) {
    rb_check_arity(argc, 1, UNLIMITED_ARGUMENTS);
    VALUE queries = rb_ary_new_from_values(argc, argv);
    rdxi_check_array_of_strings(queries);

    if (!rb_block_given_p()) {
        return rb_enumeratorize(self, rb_str_new2("fuzzy_search"), argc, argv);
    }

    void *graph;
    TypedData_Get_Struct(self, void *, &graph_type, graph);

    size_t length = (size_t)argc;
    char **converted = rdxi_str_array_to_char(queries, length);
    void *iter = rdx_graph_declarations_fuzzy_search(graph, (const char *const *)converted, length);
    rdxi_free_str_array(converted, length);

    return rdxr_graph_yield_search_results(self, iter);
}

#index_all(file_paths) ⇒ Array[String]

Returns an array of I/O error messages encountered during indexing.

Returns:

  • (Array[String])


82
83
84
85
86
87
88
89
90
91
92
93
94
95
96
97
98
99
100
101
102
103
104
105
106
107
108
109
# File 'ext/rubydex/graph.c', line 82

static VALUE rdxr_graph_index_all(VALUE self, VALUE file_paths) {
    rdxi_check_array_of_strings(file_paths);

    // Convert the given file paths into a char** array, so that we can pass to Rust
    size_t length = RARRAY_LEN(file_paths);
    char **converted_file_paths = rdxi_str_array_to_char(file_paths, length);

    // Get the underlying graph pointer and then invoke the Rust index all implementation
    void *graph;
    TypedData_Get_Struct(self, void *, &graph_type, graph);

    size_t error_count = 0;
    const char *const *errors = rdx_index_all(graph, (const char **)converted_file_paths, length, &error_count);

    rdxi_free_str_array(converted_file_paths, length);

    if (errors == NULL) {
        return rb_ary_new();
    }

    VALUE array = rb_ary_new_capa((long)error_count);
    for (size_t i = 0; i < error_count; i++) {
        rb_ary_push(array, rb_utf8_str_new_cstr(errors[i]));
    }

    free_c_string_array(errors, error_count);
    return array;
}

#index_source(uri, source, language_id) ⇒ nil

Indexes a single source string in memory, dispatching to the appropriate indexer based on language_id.

Returns:

  • (nil)


117
118
119
120
121
122
123
124
125
126
127
128
129
130
131
132
133
134
135
136
137
138
139
140
141
142
143
144
145
146
147
148
149
# File 'ext/rubydex/graph.c', line 117

static VALUE rdxr_graph_index_source(VALUE self, VALUE uri, VALUE source, VALUE language_id) {
    Check_Type(uri, T_STRING);
    Check_Type(source, T_STRING);
    Check_Type(language_id, T_STRING);

    void *graph;
    TypedData_Get_Struct(self, void *, &graph_type, graph);

    const char *uri_str = StringValueCStr(uri);
    const char *language_id_str = StringValueCStr(language_id);
    const char *source_str = RSTRING_PTR(source);
    size_t source_len = RSTRING_LEN(source);

    enum IndexSourceResult result = rdx_index_source(graph, uri_str, source_str, source_len, language_id_str);
    switch (result) {
    case IndexSourceResult_Success:
        break;
    case IndexSourceResult_InvalidUri:
        rb_raise(rb_eArgError, "invalid URI (not valid UTF-8)");
        break;
    case IndexSourceResult_InvalidSource:
        rb_raise(rb_eArgError, "source is not valid UTF-8");
        break;
    case IndexSourceResult_InvalidLanguageId:
        rb_raise(rb_eArgError, "invalid language_id (not valid UTF-8)");
        break;
    case IndexSourceResult_UnsupportedLanguageId:
        rb_raise(rb_eArgError, "unsupported language_id `%s`", language_id_str);
        break;
    }

    return Qnil;
}

#index_workspaceObject

Index all files and dependencies of the workspace that exists in workspace_path : -> Array



17
18
19
# File 'lib/rubydex/graph.rb', line 17

def index_workspace
  index_all(workspace_paths)
end

#keyword(name) ⇒ Rubydex::Keyword?

Returns the keyword object for the name, or nil if it is not a Ruby keyword.

Returns:



920
921
922
923
924
925
926
927
928
929
930
931
932
933
934
935
# File 'ext/rubydex/graph.c', line 920

static VALUE rdxr_graph_keyword(VALUE self, VALUE name) {
    Check_Type(name, T_STRING);

    const CKeyword *kw = rdx_keyword_get(StringValueCStr(name));
    if (kw == NULL) {
        return Qnil;
    }

    VALUE argv[2] = {
        rb_utf8_str_new_cstr(kw->name),
        rb_utf8_str_new_cstr(kw->documentation),
    };

    rdx_keyword_free(kw);
    return rb_class_new_instance(2, argv, cKeyword);
}

#load_config(config_path = nil) ⇒ Object

Loads a configuration file for the graph. If config_path is nil, loads the default configuration file at workspace_path/rubydex.toml if it exists. Will raise on malformed files or if an explicit path is given but the file does not exist.



888
889
890
891
892
893
894
895
896
897
898
899
900
901
902
903
904
905
906
907
908
909
910
911
912
# File 'ext/rubydex/graph.c', line 888

static VALUE rdxr_graph_load_config(int argc, VALUE *argv, VALUE self) {
    VALUE config_path;
    rb_scan_args(argc, argv, "01", &config_path);

    void *graph;
    TypedData_Get_Struct(self, void *, &graph_type, graph);

    const char *config_path_cstr = NULL;

    if (!NIL_P(config_path)) {
        Check_Type(config_path, T_STRING);
        config_path_cstr = StringValueCStr(config_path);
    }

    const char *error = rdx_graph_load_config(graph, config_path_cstr);
    if (error == NULL) {
        return Qnil;
    }

    VALUE message = rb_utf8_str_new_cstr(error);
    free_c_string(error);

    VALUE config_error = rb_const_get(mRubydex, rb_intern("ConfigError"));
    rb_exc_raise(rb_exc_new_str(config_error, message));
}

#method_referencesEnumerator[Rubydex::MethodReference]

Returns an enumerator that yields method references lazily.

Returns:



383
384
385
386
387
388
389
390
391
392
393
394
395
396
397
# File 'ext/rubydex/graph.c', line 383

static VALUE rdxr_graph_method_references(VALUE self) {
    if (!rb_block_given_p()) {
        return rb_enumeratorize_with_size(self, rb_str_new2("method_references"), 0, NULL,
                                          graph_method_references_size);
    }

    void *graph;
    TypedData_Get_Struct(self, void *, &graph_type, graph);

    void *iter = rdx_graph_method_references_iter_new(graph);
    VALUE args = rb_ary_new_from_args(2, self, ULL2NUM((uintptr_t)iter));
    rb_ensure(rdxi_method_references_yield, args, rdxi_method_references_ensure, args);

    return self;
}

#require_paths(load_paths) ⇒ Array[String]

Returns all require paths for completion.

Returns:

  • (Array[String])


546
547
548
549
550
551
552
553
554
555
556
557
558
559
560
561
562
563
564
565
566
567
568
569
570
571
# File 'ext/rubydex/graph.c', line 546

static VALUE rdxr_graph_require_paths(VALUE self, VALUE load_path) {
    rdxi_check_array_of_strings(load_path);

    void *graph;
    TypedData_Get_Struct(self, void *, &graph_type, graph);

    size_t paths_len = RARRAY_LEN(load_path);
    char **converted_paths = rdxi_str_array_to_char(load_path, paths_len);

    size_t out_count = 0;
    const char *const *results = rdx_require_paths(graph, (const char **)converted_paths, paths_len, &out_count);

    rdxi_free_str_array(converted_paths, paths_len);

    if (results == NULL) {
        return rb_ary_new();
    }

    VALUE array = rb_ary_new_capa((long)out_count);
    for (size_t i = 0; i < out_count; i++) {
        rb_ary_push(array, rb_utf8_str_new_cstr(results[i]));
    }

    free_c_string_array(results, out_count);
    return array;
}

#resolveself

Runs the resolver to compute declarations and ownership.

Returns:

  • (self)


450
451
452
453
454
455
# File 'ext/rubydex/graph.c', line 450

static VALUE rdxr_graph_resolve(VALUE self) {
    void *graph;
    TypedData_Get_Struct(self, void *, &graph_type, graph);
    rdx_graph_resolve(graph);
    return self;
}

#resolve_constant(name, nesting) ⇒ Rubydex::Declaration?

Runs the resolver on a single constant reference to determine what it points to.

Returns:



483
484
485
486
487
488
489
490
491
492
493
494
495
496
497
498
499
500
501
502
503
504
505
506
507
508
# File 'ext/rubydex/graph.c', line 483

static VALUE rdxr_graph_resolve_constant(VALUE self, VALUE const_name, VALUE nesting) {
    Check_Type(const_name, T_STRING);
    rdxi_check_array_of_strings(nesting);

    // Convert the given file paths into a char** array, so that we can pass to Rust
    size_t length = RARRAY_LEN(nesting);
    char **converted_file_paths = rdxi_str_array_to_char(nesting, length);

    void *graph;
    TypedData_Get_Struct(self, void *, &graph_type, graph);

    const CDeclaration *decl =
        rdx_graph_resolve_constant(graph, StringValueCStr(const_name), (const char **)converted_file_paths, length);

    rdxi_free_str_array(converted_file_paths, length);

    if (decl == NULL) {
        return Qnil;
    }

    VALUE decl_class = rdxi_declaration_class_for_kind(decl->kind);
    VALUE argv[] = {self, ULL2NUM(decl->id)};
    free_c_declaration(decl);

    return rb_class_new_instance(2, argv, decl_class);
}

#resolve_require_path(require_path, load_paths) ⇒ Rubydex::Document?

Resolves a require path to its document.

Returns:



516
517
518
519
520
521
522
523
524
525
526
527
528
529
530
531
532
533
534
535
536
537
538
# File 'ext/rubydex/graph.c', line 516

static VALUE rdxr_graph_resolve_require_path(VALUE self, VALUE require_path, VALUE load_paths) {
    Check_Type(require_path, T_STRING);
    rdxi_check_array_of_strings(load_paths);

    void *graph;
    TypedData_Get_Struct(self, void *, &graph_type, graph);
    const char *path_str = StringValueCStr(require_path);

    size_t paths_len = RARRAY_LEN(load_paths);
    char **converted_paths = rdxi_str_array_to_char(load_paths, paths_len);

    const uint64_t *uri_id = rdx_resolve_require_path(graph, path_str, (const char **)converted_paths, paths_len);

    rdxi_free_str_array(converted_paths, paths_len);

    if (uri_id == NULL) {
        return Qnil;
    }

    VALUE argv[] = {self, ULL2NUM(*uri_id)};
    free_u64(uri_id);
    return rb_class_new_instance(2, argv, cDocument);
}

#search(*queries) ⇒ Enumerator[Rubydex::Declaration]

Returns an enumerator that yields declarations whose name matches any of the queries exactly by substring.

Returns:



203
204
205
206
207
208
209
210
211
212
213
214
215
216
217
218
219
220
221
# File 'ext/rubydex/graph.c', line 203

static VALUE rdxr_graph_search(int argc, VALUE *argv, VALUE self) {
    rb_check_arity(argc, 1, UNLIMITED_ARGUMENTS);
    VALUE queries = rb_ary_new_from_values(argc, argv);
    rdxi_check_array_of_strings(queries);

    if (!rb_block_given_p()) {
        return rb_enumeratorize(self, rb_str_new2("search"), argc, argv);
    }

    void *graph;
    TypedData_Get_Struct(self, void *, &graph_type, graph);

    size_t length = (size_t)argc;
    char **converted = rdxi_str_array_to_char(queries, length);
    void *iter = rdx_graph_declarations_search(graph, (const char *const *)converted, length);
    rdxi_free_str_array(converted, length);

    return rdxr_graph_yield_search_results(self, iter);
}

#workspace_pathString

Returns the root directory of the workspace being indexed.

Returns:

  • (String)


851
852
853
854
855
856
857
858
859
860
861
862
# File 'ext/rubydex/graph.c', line 851

static VALUE rdxr_graph_workspace_path(VALUE self) {
    void *graph;
    TypedData_Get_Struct(self, void*, &graph_type, graph);

    const char *result = rdx_graph_workspace_path(graph);
    if (result == NULL) {
        rb_raise(rb_eRuntimeError, "Converting workspace path to Ruby string failed");
    }

    VALUE path = rdxi_owned_c_string_to_ruby(result);
    return path;
}

#workspace_path=(path) ⇒ Object

Sets the root directory of the workspace being indexed.



870
871
872
873
874
875
876
877
878
# File 'ext/rubydex/graph.c', line 870

static VALUE rdxr_graph_set_workspace_path(VALUE self, VALUE path) {
    Check_Type(path, T_STRING);

    void *graph;
    TypedData_Get_Struct(self, void*, &graph_type, graph);

    rdx_graph_set_workspace_path(graph, StringValueCStr(path));
    return path;
}

#workspace_pathsObject

Returns all workspace paths that should be indexed

: -> Array



24
25
26
27
28
29
30
31
32
33
34
35
36
37
38
39
40
# File 'lib/rubydex/graph.rb', line 24

def workspace_paths
  paths = []
  root = workspace_path

  Dir.each_child(root) do |entry|
    full_path = File.join(root, entry)

    if File.directory?(full_path) || INDEXABLE_EXTENSIONS.include?(File.extname(entry))
      paths << full_path
    end
  end

  add_workspace_dependency_paths(paths)
  add_core_rbs_definition_paths(paths)
  paths.uniq!
  paths
end