Class: Rubydex::Rules::DynamicConstantReference
- Inherits:
-
Rubydex::Rule
- Object
- Rubydex::Rule
- Rubydex::Rules::DynamicConstantReference
- Defined in:
- lib/rubydex/rules/dynamic_constant_reference.rb
Overview
Dynamic constant references cannot be reasoned about statically because they depend on runtime values. The program may still be valid, but the quality of the analysis degrades.
var::Foo
^^^^^^^^ Dynamic constant reference. This might be correct, but it cannot be understood by the analysis.
Class Method Summary collapse
-
.default_severity ⇒ Object
: -> singleton(Severity::Base).
Methods inherited from Rubydex::Rule
Class Method Details
.default_severity ⇒ Object
: -> singleton(Severity::Base)
19 20 21 |
# File 'lib/rubydex/rules/dynamic_constant_reference.rb', line 19 def default_severity Severity::Information end |